; Risset Gliss from first principles
; Iain McCurdy 2015

; iLayers       -       number of simultaneous voices. Should be an integer.
; kIntvl        -       interval between adjacent layers (in semitones)
; kCentre       -       central note of the texture (as a note number)
; kPhase        -       global control of the phase of the gliss. Should be a moving value within the range 0 to 1.       
; iRndPhs	-	amplitude of random phase for each partial. 0 = no randomness, 1 = maximum randomness
; giAmpFn       -       amplitude scaling function applied across the range of the texture 
; giWfm         -       waveform used by the oscillator

<CsoundSynthesizer>

<CsOptions>
-odac -dm0
</CsOptions>

<CsInstruments>

0dbfs     =            1                                                                                                                

giWfm     ftgen        0,0,4097,10,1
giAmpFn  ftgen         0,0,1024,-20,6,1,1.2 ; gaussian window

opcode EndlessGlissSynth,a,kkkiiiip
 kPhase,kCentre,kIntvl,iAmpFn,iWfm,iRndPhs,iLayers,iCount  xin
 kLPhs    wrap         kPhase+((iCount-1)/iLayers),0,1  ; calculate phase for this layer (and wrap within the range 0 to 1) 
 kAmp     tablei       kLPhs,iAmpFn,1                   ; read amplitude from spectral envelope
 kRange   =            iLayers * kIntvl                 ; complete texture range (in semitones)
 kNote    limit        kCentre - (kRange*0.5) + (kRange * kLPhs),0,127 ; calculate note (number)
 kAmpScl  scale        logcurve:k(kNote/127,4),0.01,1   ; amplitude scaling. (attenuate higher notes)
 a1       poscil       kAmp*kAmpScl,cpsmidinn(kNote),iWfm ,rnd(iRndPhs)
 if iCount<iLayers then                                 ; call next layer (if required)
  amix    EndlessGlissSynth kPhase,kCentre,kIntvl,iAmpFn,iWfm,iRndPhs,iLayers,iCount+1
 endif
          xout         a1 + amix                        ; return audio to caller instrument
endop

instr   1
 kAmp     =            p4
 iLayers  =            p5    ; number of layers
 kIntvl   init         p6    ; gap between adjacent layers
 kCentre  init         p7    ; central pitch as a note number
 kPhase   phasor       p8/iLayers
 iRndPhs  =            p9    ; random initial phase amount
 aMix     EndlessGlissSynth kPhase,kCentre,kIntvl,giAmpFn,giWfm,iRndPhs,iLayers
 aEnv     linsegr      0,0.01,1,0.01,0
          out          aMix*kAmp*aEnv
endin

</CsInstruments>

<CsScore>
;----------amp--layers--intvl--centre--rate--RndPhs
i 1 0  14  0.7     6      12    60     0.15    0
i 1 15 14  0.2    40       3    60     0.2     0
i 1 30 14  0.2    40     0.5    72     2       0
i 1 45 14  0.2    40     0.5    72     2       1
i 1 60 29  0.07  112       2    60     -0.3    1
</CsScore>

</CsoundSynthesizer>
