; Risset Gliss Signal Processing
; Iain McCurdy, 2015.

; aSig          -       input signal
; kPhase        -       global control of the phase of the gliss. Should be a moving value within the range 0 to 1.    
; kCentre       -       central note of the texture (as a note number)
; kIntvl        -       interval between adjacent layers (in semitones)
; giAmpFn       -       amplitude scaling function applied across the complete range of the texture 
; iLayers       -       number of simultaneous voices. Should be an integer.

<CsoundSynthesizer>

<CsOptions>
-odac -dm0
</CsOptions>

<CsInstruments>

0dbfs      =         1                                                                                                                 

giAmpFn    ftgen     0,0,1024,7,0,512,1,512,0 ; triangle

opcode EndlessGlissComb,a,akkkiip
 aSig,kPhase,kCentre,kIntvl,iAmpFn,iLayers,iCount  xin
 kLPhs     wrap      kPhase+((iCount-1)/iLayers),0,1     ; phase position for this layer
 kAmp      tablei    kLPhs,iAmpFn,1                      ; read amplitude from spectral envelope
 kRange   =            iLayers * kIntvl                 ; complete texture range (in semitones)
 kNote    limit        kCentre - (kRange*0.5) + (kRange * kLPhs),0,127 ; calculate note (number)
 
 adel      interp    1/cpsmidinn(kNote)                  ; convert pitch to delay time
 aRes      flanger   aSig, adel, 0.999                   ; comb filter (flanger)
 aRes      dcblock2  aRes                                ; remove DC offset
 
 if iCount<iLayers then                                  ; create next layer (if required)
  aMix EndlessGlissComb aSig,kPhase,kCentre,kIntvl,iAmpFn,iLayers,iCount+1
 endif
           xout      (aRes*kAmp) + aMix                  ; return this layer audio and all subsequent layers to caller instrument
endop

instr   1
 kAmp      =         0.2
 iLayers   =         24                               ; number of layers
 kIntvl    init      3                                ; gap between adjacent layers
 kCentre   init      60                               ; central pitch as a note number
 kPhase    phasor    -0.05/iLayers

 ; generate some audio
 aSig      dust2     0.5,10                           ; generate some  audio dust
 aSig      butlp     aSig,cpsoct(randomi:k(6,12,8,1)) ; random LP filtering

 aMix      EndlessGlissComb  aSig,kPhase,kCentre,kIntvl,giAmpFn,iLayers
           out       aMix
endin

</CsInstruments>

<CsScore>
i 1 0 300
</CsScore>

</CsoundSynthesizer>
